<?php

/**
-------------------------------------------------------------------------
mediamallfactory - Media Mall Factory 4.2.2
-------------------------------------------------------------------------
 * @author thePHPfactory
 * @copyright Copyright (C) 2011 SKEPSIS Consult SRL. All Rights Reserved.
 * @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Websites: http://www.thePHPfactory.com
 * Technical Support: Forum - http://www.thePHPfactory.com/forum/
-------------------------------------------------------------------------
*/

defined('_JEXEC') or die;

class FactoryMailer
{
    protected $mailer;

    public function __construct()
    {
        $this->mailer = JFactory::getMailer();
    }

    public static function getInstance()
    {
        static $instance = null;

        if (is_null($instance)) {
            $instance = new self();
        }

        return $instance;
    }

    public function send($type, $receiverId, $variables = array(), $isHtml = true)
    {
        // Initialise variables.
        $receiver = JFactory::getUser($receiverId);
        $app = JFactory::getApplication();
        $receiverLanguage = $receiver->getParam('language', JComponentHelper::getParams('com_languages')->get('site'));
        $receiverEmail = $receiver->email;
        $options = $this->getNotificationOptions($type);
        $notification = $this->getNotification($type, $receiverLanguage);

        // Check if notification was found.
        if (!$notification) {
            return false;
        }

        // Prepare subject and body.
        $subject = $this->parseVariables($notification->subject, $options, $variables);
        $body = $this->parseVariables($notification->body, $options, $variables);

        // Send mail.
        $this->mailer->setSubject($subject);
        $this->mailer->setBody($body);
        $this->mailer->addRecipient($receiverEmail);
        $this->mailer->setSender(array($app->get('mailfrom'), $app->get('fromname')));
        $this->mailer->isHtml($isHtml);

        if (!$this->mailer->send()) {
            return false;
        }

        return true;
    }

    protected function getNotificationOptions($type)
    {
        $options = array();
        $xml = simplexml_load_file(FactoryApplication::getInstance()->getPath('component_administrator') . DS . 'notifications.xml');
        $notification = $xml->xpath('//notification[@type="' . $type . '"]');

        if (!$notification) {
            return $options;
        }

        foreach ($notification[0]->option as $option) {
            $options[] = (string)$option;
        }

        return $options;
    }

    protected function getNotification($type, $receiverLanguage)
    {
        $dbo = JFactory::getDbo();
        $query = $dbo->getQuery(true)
            ->select('n.*')
            ->from('#__mediamallfactory_notifications n')
            ->where('n.type = ' . $dbo->quote($type))
            ->where('n.published = ' . $dbo->quote(1))
            ->where('(n.lang_code = ' . $dbo->quote($receiverLanguage) . ' OR n.lang_code = ' . $dbo->quote('*') . ')');
        $notifications = $dbo->setQuery($query)
            ->loadObjectList('lang_code');

        if (!$notifications) {
            return false;
        }

        return isset($notifications[$receiverLanguage]) ? $notifications[$receiverLanguage] : $notifications['*'];
    }

    protected function parseVariables($string, $search, $replace)
    {
        // Prepare variables.
        foreach ($search as &$variable) {
            $variable = '%%' . $variable . '%%';
        }

        // Replace variables.
        $string = str_replace($search, $replace, $string);

        // Replace image sources.
        $string = str_replace('src="', 'src="' . JURI::root(), $string);

        return $string;
    }
}
